// FOR_YOU:
// This is the main source of your plugin.
// It's in this source you'll type all your assembly or c++ code and all 
// the variables and memory zones you need.

// ************  Top/Header section ************
#include "stdafx.h"
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <limits.h>
#include "bass.h"		// prototypes for extra sound library: bass.dll
	// Following header files will be updated for every new version of 
	// the tomb_NextGeneration.dll, so it's better you don't change them
	//  because they will be replaced for any new update.

#include "Tomb_NextGeneration.h" // mnemonic constants defined in tomb_NextGeneration.dll
#include "structures.h" // structure of tomb4 program and trng dll
#include "DefTomb4Funct.h" // defines of tomb4 procedure prototypes
#include "functions.h"  // assigments of real tomb4 address to each tomb4 procedures
#include "macros.h"  // definition of macros

	// FOR_YOU:
	// While the followings are the headers you can use 
	// to type your structures, constants and new tomb4 procedures you 
	// discovered. 
	// Following files are only yours and trng will not ever change them:
#include "macros_mine.h"  // here you define your own macros
#include "constants_mine.h" // here you define your mnemonic constants
#include "structures_mine.h" // here you type your structure definitions
#include "Tomb4Discoveries_mine.h" // here type tomb4 procedures you discovered

#include "trng.h" // list of trng functions imported from trng.cpp source. 

#pragma warning( error : 4706 )
#pragma warning(disable: 4996)

// ************  Early function declarations ************

 
// ************  Global Variables Section *************

// FOR_YOU:
// here you type the variables or memory zones you wish use in different
// C++ procedures.
// If you use the assembly you'll have to type ALL your variables in this
// section.
// for example if you wish have a 32 bits signed variable to store big
// numbers you can type:
// TYPE_HERE:
// int MyNumber;
// and then you can use it in asm in this way:
//		mov  eax, [MyNumber]
// or
//      mov eax, dword ptr [MyNumber]
// same speech for wide memory zones.
// If you wish having a memory zone of 13000 bytes you can type:
//  BYTE MyZone[13000];
// and then you can use it in this way:
//     mov  al, MyNumber[ecx]  ; read the ECXth byte and copy it in AL
// or also:
//     mov al, [MyNumber+ecx] ; same result of above instruction

// Variables and memory zone to TYPE_HERE:

HINSTANCE MyDllInstance=NULL;  // the instance handle of your dll

extern char BufferLog[4096]; // temporary global buffer to host error and warning messages

// FOR_YOU: If you mean create code patch (changes in tomb4 code to call your code in the plugin dll,
// you MUST change the MyTomb4PatcherAddress value (here below) 
// You can discover a free tomb4 zone using TrngPatcher program, with the 
// menu command "Tools->Get Assignment of Free Memory range"
// Note: choose an address, in the given range, terminating with hex 
// digits: 0,4,8, or C
// because it's usefull that the address was a multiple by 4
// If you chose an address used from other plugins you'll get an error and
// the game will be aborted
// note: if you don't mean use code patches you can let 0x0 in following line
DWORD MyTomb4PatcherAddress = 0x0; // <- TYPE_HERE: the new address you chose
								
// this text will contain your plugin name (omitting .dll extension).
// it will be used to give a title to messagebox for error messages or warnings
char TexMyPluginName[80];  
// in this MyData structure you can store all global variables for your plugin
// you have to define them in structures_mine.h source inside structure "StrMyData" or for variable
// to save and restore in savegames, in "StrSavegameGlobalData" or "StrSavegameLocalData" structures
StrMyData MyData;

char **const SavegameFilenameFormat = (char **) SAVEGAME_FILENAME_FORMAT_ADDRESS;

WORD SaveCrystalBlueSlot, SaveCrystalGreenSlot, FallingFloorSlot, TR1SpikesSlot;
short SaveCrystalBlueUseSound, SaveCrystalGreenCollectSound, TimeTolerance, HealthLossPerFrame, DeathAnimation;
bool SaveCrystalBluePreciseAnimation, QuickSave, SaveCrystalGreenPrecisePosition, InstantKill, ImpalementAfterHealthDepletion, ImpalementAfterLanding;
StrSingleColors SaveCrystalBlueColour, SaveCrystalGreenColour;

// ************  Utilities section  ****************

int Modulus(int Number, int Base)
{
	Number %= Base;
	return Number >= 0 ? Number : Number + Base;
}

int AbsoluteFacing(StrItemTr4 *Source, StrItemTr4 *Target)
{
	int Angle, Explement;

	Angle = Modulus(mGetAngle(Source->CordX, Source->CordZ, Target->CordX, Target->CordZ) + ANGLE_CORRECTION - Source->OrientationH, USHRT_MAX + 1);
	Explement = USHRT_MAX - Angle + 1;
	return Angle < Explement ? Angle : Explement;
}

DWORD MaximumDistance(StrItemTr4 *Source, StrItemTr4 *Target)
{
	DWORD DifferenceXAxis, DifferenceZAxis;
	int DifferenceYAxis;

	DifferenceXAxis = Source->CordX > Target->CordX ? Source->CordX - Target->CordX : Target->CordX - Source->CordX;
	DifferenceYAxis = Source->CordY > Target->CordY ? Source->CordY - Target->CordY : Target->CordY - Source->CordY;
	DifferenceZAxis = Source->CordZ > Target->CordZ ? Source->CordZ - Target->CordZ : Target->CordZ - Source->CordZ;
	if (DifferenceXAxis > DifferenceZAxis) {
		return DifferenceXAxis > (DWORD) DifferenceYAxis ? DifferenceXAxis : DifferenceYAxis;
	} else {
		return DifferenceZAxis > (DWORD) DifferenceYAxis ? DifferenceZAxis : DifferenceYAxis;
	}
}

bool SameSector(DWORD SourceX, DWORD SourceZ, DWORD TargetX, DWORD TargetZ)
{
	return SourceX - SourceX % DISTANCE_SECTOR == TargetX - TargetX % DISTANCE_SECTOR
			&& SourceZ - SourceZ % DISTANCE_SECTOR == TargetZ - TargetZ % DISTANCE_SECTOR;
}

// ************  Patcher Functions section  ***************
// Note: please, don't change or remove the C++ procedure you find here,
// because they are necessary to do work the plugin with the trng dll
// Anyway in many of these functions you can add your code


// FOR_YOU: In this function you insert the callings of functions used to change tomb4 code
// You can get these functions, in the correct format, using Trng Core -> Asm Souce Editor -> Debugging menu
// program
// note: if there is a conflict the function will return "false"
// and the tomb4 program will abort with an error message
bool CreateMyCodePatches(void)
{
	// the call of the code patch to TYPE_HERE:
	// example: 
	// SET_PATCH(Path_RedirCollision)
	// to call the function Patch_RedirCollision() created with TrngPatcher program (command Assmembly->Create Dynamic Patch Generator)

	return true;
}

// ************  Assembly Procedures section  ******************

// FOR_YOU: In the SubPatchArray you'll type all procedure names of your code in the dll you
// wish were called from tomb4 code.
// type them in the order of ax value. So first asm proc in the list, will be called
// with ax=0, while the second in the list will be called with ax=1 ect.



void *SubPatchArray[] = {
	
// TYPE_HERE your asm procedure names to call from tomb4 code
	NULL
};


// ************  MainPatcher() asm procedure  *****************

// FOR_YOU: This is your main patcher procedure.
// All your patch in tomb4 code will call this procedure passing to it, in ax register,
// the number of subpatch to call

BEGIN_ASM_PROC(MainPatcher)
	and eax, 0ffffh
	mov eax, dword ptr [SubPatchArray+eax*4];
	jmp eax
END_ASM_PROC



// ************  CallBack functions section  *****************

// TYPE_HERE: your callback function

void cbInitProgram(int NumberLoadedPlugins, char *VetPluginNames[]) 
{

	// save number of plugins (enclosed yours and the tomb_nextgeneration.dll ) and their names
	// these data will be used to locate IDs of any loaded plugins using FindPluginID() function
	Trng.TotPlugins = NumberLoadedPlugins;
	Trng.pVetPluginNames = VetPluginNames;

	// clear all my global variables
	ClearMemory(&MyData, sizeof(StrMyData));
}


void cbInitGame(void)
{
	// here you can initialize your global data for whole adventure
	// this procedure will be called only once, before loading title level


}

void cbInitLevel(void)
{
	// here you can initialize specific items of currnet level.
	// it will be called only once for level, when all items has been already initialized
	// and just a moment before entering in main game cycle.

	if (SaveCrystalGreenSlot ? !QuickSave || !MyData.Save.Global.SaveCrystalGreenQuantity : SaveCrystalBlueSlot)
		MyData.Save.Global.ClearCommand = true;
}

// called everytime player save the game (but also when lara move from a level to another HUB saving). 
// in this procedure your plugin will save its own data in savegame
// SavingType inform about the kind of saving it is happening (SAVT_... values)
// pAdrZone is a byte pointer where this procedure will save the address from where begin data to save in savegame
// this function will return the size of data to save
// note: It's better you don't change anything of this code. It will save byself of data you stored in MyData.Save
// structure. If you wish adding other variables to save, just you add new fields in Structures of MyData.Save 
DWORD cbSaveMyData(BYTE **pAdrZone, int SavingType)
{
	DWORD SizeData;
	int i;
	static WORD *pVetExtras;
	int TotNWords;
	int TotNewActions;


	if (SavingType & SAVT_COMPLETED) {
		// this call is not to save data but only it is a confirm that the previous saving has been completed
		// now we can free the temporary memory used to save the data in previous call
		if (pVetExtras != NULL) {
			FreeMemory(pVetExtras);
			pVetExtras=NULL;
		}

		return 0;
	}


	TotNWords=0;
	pVetExtras = (WORD *) GetMemory(16);
	// save id of my plugin in first word

	pVetExtras[TotNWords++] = Trng.IdMyPlugin;

	if (SavingType & SAVT_LOCAL_DATA) {
		// save local data

		// save Local structure
		AddNGToken(NGTAG_LOCAL_DATA, NO_ARRAY, sizeof(StrSavegameLocalData), &MyData.Save.Local, 
						&pVetExtras, &TotNWords);


		// save all (currently enabled) progressive actions
		// before saving, compact progressive action array to remove intermediate free records
		TotNewActions=0;

		for (i=0;i<MyData.TotProgrActions;i++) {
			if (MyData.VetProgrActions[i].ActionType != AXN_FREE) {

				MyData.VetProgrActions[TotNewActions] = MyData.VetProgrActions[i];

				TotNewActions++;
			}
		}
		// update new valuese after recompatting
		MyData.LastProgrActionIndex =0;
		MyData.TotProgrActions= TotNewActions;

		// store all progressive action records
		AddNGToken(NGTAG_PROGRESSIVE_ACTIONS, MyData.TotProgrActions, sizeof(StrProgressiveAction), 
				&MyData.VetProgrActions[0], &pVetExtras, &TotNWords);

	}

	if (SavingType & SAVT_GLOBAL_DATA) {
		// save global data
		AddNGToken(NGTAG_GLOBAL_DATA, NO_ARRAY, sizeof(StrSavegameGlobalData), &MyData.Save.Global , 
						&pVetExtras, &TotNWords);
	}
	// write final sequence
	AddTokenFinalSequence(&pVetExtras, &TotNWords);

	// return to trng the infos about start of memory where there are our data and their size:
	*pAdrZone = (BYTE *) pVetExtras;
	SizeData = TotNWords * 2;

	return SizeData;
	

}
// called when a savegame will be loaded (but also when lara move from a level to another)
// pAdrZone will point to memory zone with data just loaded from savegame
// SizeData is the size of data pointed by pAdrZone
// note: it's better you don't change anything of this code. It's alread thought to reload all data you saved in MyData.Save 
// structure. There is no need of changes by you
void cbLoadMyData(BYTE *pAdrZone, DWORD SizeData)
{

	WORD *pVetExtras;
	StrParseNGField  ParseField;
	int Indice;
	int i;
	WORD TotActions;

	
	pVetExtras = (WORD*) pAdrZone;

	Indice=0;

	while (ParseNgField(pVetExtras ,Indice, &ParseField)==true) {
		
		// recover different ng token
		switch (ParseField.Type) {
		case NGTAG_LOCAL_DATA:
			// local data
			memcpy(&MyData.Save.Local, ParseField.pData, sizeof(StrSavegameLocalData));
			break;

		case NGTAG_GLOBAL_DATA:
			// global data
			memcpy(&MyData.Save.Global, ParseField.pData, sizeof(StrSavegameGlobalData));
			break;
			
		case NGTAG_PROGRESSIVE_ACTIONS:
			// progressive actions
			i= ParseField.StartDataIndex;
			// read tot actions value
			TotActions = pVetExtras[i++];
			// copy all tot records
			memcpy(&MyData.VetProgrActions[0], &pVetExtras[i], sizeof(StrProgressiveAction) * TotActions);
			MyData.TotProgrActions = TotActions;
			break;
		}
		Indice= ParseField.NextIndex; 
	}

}
// free memory used to store all data about your customize commands loaded in previous level
void FreeMemoryCustomize(void)
{
	int i;

	for (i=0;i<MyData.BaseCustomizeMine.TotCustomize;i++) {
		FreeMemory(MyData.BaseCustomizeMine.pVetCustomize[i].pVetArg);
	}

	if (MyData.BaseCustomizeMine.TotCustomize > 0) {
		FreeMemory(MyData.BaseCustomizeMine.pVetCustomize);
		MyData.BaseCustomizeMine.TotCustomize=0;
	}


	MyData.BaseCustomizeMine.pVetCustomize=NULL;
}

// free memory used to store all data about your parameters commands loaded in previous level
void FreeMemoryParameters(void)
{
	int i;

	for (i=0;i<MyData.BaseParametersMine.TotParameters;i++) {
		FreeMemory(MyData.BaseParametersMine.pVetParameters[i].pVetArg);
	}

	if (MyData.BaseParametersMine.TotParameters > 0) {
		FreeMemory(MyData.BaseParametersMine.pVetParameters);
		MyData.BaseParametersMine.TotParameters=0;
	}

	MyData.BaseParametersMine.pVetParameters=NULL;
}

// this procedure will be called at end of any level
// you can type here code to free resources allocated for level (that quits now)
void FreeLevelResources(void)
{

	// free memory used to store all data about your customize commands loaded in previous level
	FreeMemoryCustomize();
	// free memory used to store all data about your parameters commands loaded in previous level
	FreeMemoryParameters();
	MyData.BaseAssignSlotMine.TotAssign=0;

}
// it will be called before beginning the loading for a new level.
// you can type here code to initialise all variables used for level (to clear old values changed by previous level)
// and to free resources allocated in old level since now we'are going to another new level.

void cbInitLoadNewLevel(void)
{
	int i;

	StrProgressiveAction *pAction;

	// clear all LOCAL variables
	ClearMemory(&MyData.Save.Local,sizeof(StrSavegameLocalData));

	// clear progressive actions
	pAction= &MyData.VetProgrActions[0];

	for (i=0;i<MyData.TotProgrActions;i++) {
		if (pAction->ActionType != AXN_FREE) {
			// here you could analise to free resoruce allocated from this specific action

			pAction->ActionType = AXN_FREE;
		}
	}

	MyData.TotProgrActions=0;
	MyData.LastProgrActionIndex=0;

	// here you can initialise other variables of MyData different than Local and progressive actions
	// free resources allocate in previous level
	FreeLevelResources();

	SaveCrystalBlueSlot = 0;
	SaveCrystalGreenSlot = 0;
	FallingFloorSlot = 0;
	TR1SpikesSlot = 0;
	SaveCrystalBluePreciseAnimation = true;
	SaveCrystalBlueUseSound = INDEX_NONE;
	SaveCrystalBlueColour.Red = 0;
	SaveCrystalBlueColour.Green = 0;
	SaveCrystalBlueColour.Blue = 0;
	QuickSave = true;
	SaveCrystalGreenPrecisePosition = true;
	SaveCrystalGreenCollectSound = INDEX_NONE;
	SaveCrystalGreenColour.Red = 0;
	SaveCrystalGreenColour.Green = 0;
	SaveCrystalGreenColour.Blue = 0;
	TimeTolerance = 0;
	HealthLossPerFrame = HEALTH_LOSS_DEFAULT;
	InstantKill = true;
	ImpalementAfterHealthDepletion = true;
	ImpalementAfterLanding = true;
	DeathAnimation = ANIMATION_IMPALED_ON_SPIKES;
	MyData.Save.Global.SaveCrystalBlueIndex = INDEX_NONE;
	MyData.Save.Global.SaveCrystalGreenQuantity = 0;
	MyData.Save.Global.Guard = false;
	MyData.Save.Global.SendCommand = false;
	MyData.Save.Global.ClearCommand = false;
}



// this procedure will be called everytime a flipeffect of yours will be engaged
// you have to elaborate it and then return a TRET_.. value (most common is TRET_PERFORM_ONCE_AND_GO)
int cbFlipEffectMine(WORD FlipIndex, WORD Timer, WORD Extra, WORD ActivationMode)
{
	int RetValue;
	WORD TimerFull;

	RetValue = enumTRET.PERFORM_ONCE_AND_GO;
	// if the flip has no Extra paremeter you can handle a Timer value with values upto 32767
	// in this case you'll use the following TimerFull variable, where (with following code) we set a unique big number 
	// pasting togheter the timer+extra arguments:
	TimerFull = Timer | (Extra << 8);

	switch (FlipIndex) {
		// here type the "case Number:" for each flipeffect number. At end of the code you'll use the "break;" instruction to signal the code ending
		// Note: when you'll add your first "case Number:" then you can remove the following "case -1: and break;" instructions
	case FLIPEFFECT_GET_SAVECRYSTAL_GREEN_QUANTITY:
		if (ActivationMode & SCANF_SCRIPT_TRIGGER && SaveCrystalGreenSlot)
			Trng.pGlobTomb4->pBaseVariableTRNG->Globals.CurrentValue = MyData.Save.Global.SaveCrystalGreenQuantity;
		break;
	case FLIPEFFECT_SET_SAVECRYSTAL_GREEN_QUANTITY:
		if (ActivationMode & SCANF_SCRIPT_TRIGGER
				&& SaveCrystalGreenSlot
				&& Trng.pGlobTomb4->pBaseVariableTRNG->Globals.CurrentValue >= 0
				&& Trng.pGlobTomb4->pBaseVariableTRNG->Globals.CurrentValue <= QUANTITY_SAVECRYSTAL_GREEN_MAXIMUM) {
			MyData.Save.Global.SaveCrystalGreenQuantity = Trng.pGlobTomb4->pBaseVariableTRNG->Globals.CurrentValue;
			MyData.Save.Global.ClearCommand = !QuickSave || !MyData.Save.Global.SaveCrystalGreenQuantity;
		}
		break;
	default:
		SendToLog("WARNING: Flipeffect trigger number %d has not been handled in cbFlipEffectMine() function", FlipIndex);
		break;
	}

	// if there was the one-shot button enabled, return TRET_PERFORM_NEVER_MORE
	if (ActivationMode & enumSCANF.BUTTON_ONE_SHOT) RetValue= enumTRET.PERFORM_NEVER_MORE; 
	return RetValue;
}
// this procedure will be called everytime an action trigger of yours will be engaged 
// you have to elaborate it and then return a TRET_.. value (most common is TRET_PERFORM_ONCE_AND_GO)
int cbActionMine(WORD ActionIndex, int ItemIndex, WORD Extra, WORD ActivationMode)
{
	int RetValue;
	
	RetValue=TRET_PERFORM_ONCE_AND_GO;

	switch (ActionIndex) {
		// type here the code per your action trigger.
		// add "case Number:" and complete the code with "break;" instruction
	case -1:
		// note: remove this "case -1:" and its "break;" it has been added only to avoid warning messages about empty switch
		break;
	default:
		SendToLog("WARNING: action trigger number %d has not been handled in cbActionMine() function", ActionIndex);
		break;
	}
	
	// if there was the one-shot button enabled, return TRET_PERFORM_NEVER_MORE
	if (ActivationMode & enumSCANF.BUTTON_ONE_SHOT) RetValue= enumTRET.PERFORM_NEVER_MORE;
	return RetValue;


}

// this procedure will be called everytime a conditional trigger of yours will be engaged
// you have to elaborate it and then return a CTRET_.. value (most common is CTRET_ONLY_ONCE_ON_TRUE)
int cbConditionMine(WORD ConditionIndex, int ItemIndex, WORD Extra, WORD ActivationMode)
{
	int RetValue;
	
	RetValue=CTRET_ONLY_ONCE_ON_TRUE;

	switch (ConditionIndex){
		// type here the code for your condition trigger, inserting the code in the section
		// beginning with "case NumberOfAction:" and ending with row "break;"
	case -1:
		// note: remove this "case -1:" and its "break;" it has been added only to avoid warning messages about empty switch
		break;
	default:
		SendToLog("WARNING: condition trigger number %d has not been handled in cbConditionMine() function", ConditionIndex);
		break;


	}
	return RetValue;
	  
}

// this procedure vill be called for each Customize=CUST_... command read from script
// having one of yours CUST_ constant
// CustomizeValue will be the value of your CUST_ constant
// NumberOfItems will be the number of following Item (signed 16 bit values) following
// the CUST_ constant in the customize= script command
// pItemArray is the array with all NumberOfItems arguments of customize command
void cbCustomizeMine(WORD CustomizeValue, int NumberOfItems, short *pItemArray)
{
	// here you can replace this default management of anonymous customize commands
	// with your procedure where you can recognize each different CUST_ value and 
	// save its arguments in meaningful names fields, or elaboriting them immediatly
	// when it is possible (warning: in this moment nothing of level it has been yet loaded, excepting the script section)

	// ----- default management (optional)----
	// all customize values will be saved in MyData structure
	DWORD SizeMem;
	StrGenericCustomize *pMyCust;
	int TotCust;

	// ask memory to have another (new) record of StrGenericCustomize structure
	TotCust= MyData.BaseCustomizeMine.TotCustomize;
	TotCust++;
	SizeMem = TotCust * sizeof(StrGenericCustomize);
	MyData.BaseCustomizeMine.pVetCustomize = 
				(StrGenericCustomize *) ResizeMemory(MyData.BaseCustomizeMine.pVetCustomize, SizeMem);

	pMyCust = & MyData.BaseCustomizeMine.pVetCustomize[TotCust-1];
	
	// now require memory for all arguments (NumberOfItems) store in pItemArray

	pMyCust->pVetArg = (short *) GetMemory(2 * NumberOfItems);
	// copy data
	pMyCust->NArguments = NumberOfItems;
	memcpy(pMyCust->pVetArg, pItemArray, 2*NumberOfItems);
	pMyCust->CustValue = CustomizeValue;

	MyData.BaseCustomizeMine.TotCustomize= TotCust;
	// ---- end of default managemnt for generic customize -------------	

	switch (CustomizeValue) {
	case CUST_SAVECRYSTAL_BLUE:
		if (NumberOfItems == ARGUMENT_COUNT_SAVECRYSTAL_BLUE) {
			SaveCrystalBluePreciseAnimation = !!pItemArray[ARGUMENT1];
			SaveCrystalBlueUseSound = pItemArray[ARGUMENT2] >= 0 ? pItemArray[ARGUMENT2] : INDEX_NONE;
			if (pItemArray[ARGUMENT3] >= 0
					&& pItemArray[ARGUMENT3] <= UCHAR_MAX
					&& pItemArray[ARGUMENT4] >= 0
					&& pItemArray[ARGUMENT4] <= UCHAR_MAX
					&& pItemArray[ARGUMENT5] >= 0
					&& pItemArray[ARGUMENT5] <= UCHAR_MAX) {
				SaveCrystalBlueColour.Red = (BYTE) pItemArray[ARGUMENT3];
				SaveCrystalBlueColour.Green = (BYTE) pItemArray[ARGUMENT4];
				SaveCrystalBlueColour.Blue = (BYTE) pItemArray[ARGUMENT5];
			} else {
				SaveCrystalBlueColour.Red = 0;
				SaveCrystalBlueColour.Green = 0;
				SaveCrystalBlueColour.Blue = 0;
			}
		} else {
			SaveCrystalBluePreciseAnimation = true;
			SaveCrystalBlueUseSound = INDEX_NONE;
			SaveCrystalBlueColour.Red = 0;
			SaveCrystalBlueColour.Green = 0;
			SaveCrystalBlueColour.Blue = 0;
		}
		break;
	case CUST_SAVECRYSTAL_GREEN:
		if (NumberOfItems == ARGUMENT_COUNT_SAVECRYSTAL_GREEN) {
			QuickSave = !!pItemArray[ARGUMENT1];
			SaveCrystalGreenPrecisePosition = !!pItemArray[ARGUMENT2];
			SaveCrystalGreenCollectSound = pItemArray[ARGUMENT3] >= 0 ? pItemArray[ARGUMENT3] : INDEX_NONE;
			if (pItemArray[ARGUMENT4] >= 0
					&& pItemArray[ARGUMENT4] <= UCHAR_MAX
					&& pItemArray[ARGUMENT5] >= 0
					&& pItemArray[ARGUMENT5] <= UCHAR_MAX
					&& pItemArray[ARGUMENT6] >= 0
					&& pItemArray[ARGUMENT6] <= UCHAR_MAX) {
				SaveCrystalGreenColour.Red = (BYTE) pItemArray[ARGUMENT4];
				SaveCrystalGreenColour.Green = (BYTE) pItemArray[ARGUMENT5];
				SaveCrystalGreenColour.Blue = (BYTE) pItemArray[ARGUMENT6];
			} else {
				SaveCrystalGreenColour.Red = 0;
				SaveCrystalGreenColour.Green = 0;
				SaveCrystalGreenColour.Blue = 0;
			}
		} else {
			QuickSave = true;
			SaveCrystalGreenPrecisePosition = true;
			SaveCrystalGreenCollectSound = INDEX_NONE;
			SaveCrystalGreenColour.Red = 0;
			SaveCrystalGreenColour.Green = 0;
			SaveCrystalGreenColour.Blue = 0;
		}
		break;
	case CUST_FALLING_FLOOR:
		TimeTolerance = NumberOfItems == ARGUMENT_COUNT_FALLING_FLOOR && pItemArray[ARGUMENT1] >= 0 ? pItemArray[ARGUMENT1] : 0;
		break;
	case CUST_TR1_SPIKES:
		if (NumberOfItems == ARGUMENT_COUNT_TR1_SPIKES) {
			HealthLossPerFrame = pItemArray[ARGUMENT1] >= 0 ? pItemArray[ARGUMENT1] : HEALTH_LOSS_DEFAULT;
			InstantKill = !!pItemArray[ARGUMENT2];
			ImpalementAfterHealthDepletion = !!pItemArray[ARGUMENT3];
			ImpalementAfterLanding = !!pItemArray[ARGUMENT4];
			DeathAnimation = pItemArray[ARGUMENT5] >= 0 ? pItemArray[ARGUMENT5] : ANIMATION_IMPALED_ON_SPIKES;
		} else {
			HealthLossPerFrame = HEALTH_LOSS_DEFAULT;
			InstantKill = true;
			ImpalementAfterHealthDepletion = true;
			ImpalementAfterLanding = true;
			DeathAnimation = ANIMATION_IMPALED_ON_SPIKES;
		}
		break;
	}
}
// callback called everytime in current level section of the script it has been found an AssignSlot command
// with one of your OBJ_ constants
void cbAssignSlotMine(WORD Slot, WORD ObjType)
{
	int i;

	i = MyData.BaseAssignSlotMine.TotAssign;

	if (i >= MAX_ASSIGN_SLOT_MINE) {
		SendToLog("ERROR: too many AssignSlot= commands for current plugin");
		return;
	}

	MyData.BaseAssignSlotMine.VetAssignSlot[i].MioSlot = Slot;
	MyData.BaseAssignSlotMine.VetAssignSlot[i].TipoSlot = ObjType;
	MyData.BaseAssignSlotMine.TotAssign++;

	if (Slot >= SLOT_NUMBER_OBJECTS)
		Slot = 0;

	switch (ObjType) {
	case OBJ_SAVECRYSTAL_BLUE:
		SaveCrystalBlueSlot = Slot;
		break;
	case OBJ_SAVECRYSTAL_GREEN:
		SaveCrystalGreenSlot = Slot;
		break;
	case OBJ_FALLING_FLOOR:
		FallingFloorSlot = Slot;
		break;
	case OBJ_TR1_SPIKES:
		TR1SpikesSlot = Slot;
		break;
	}
}
// this procedure vill be called for each Parameters=PARAM_... command read from script
// having one of yours PARAM_ constants
// ParameterValue will be the value of your PARAM_ constant
// NumberOfItems will be the number of following Item (signed 16 bit values) following
// the PARAM_ constant in the customize= script command
// pItemArray is the array with all NumberOfItems arguments of Parameter command
void cbParametersMine(WORD ParameterValue, int NumberOfItems, short *pItemArray)
{
	// here you can replace this default management of anonymous parameters commands
	// with your procedure where you can recognize each different Param_ value and 
	// save its arguments in meaningful names fields, or elaboriting them immediatly
	// when it is possible (warning: in this moment nothing of level it has been yet loaded, excepting the script section)

	// ----- default management (optional)----
	// all parameters values will be saved in MyData structure
	DWORD SizeMem;
	StrGenericParameters *pMyParam;
	int TotParam;

	// ask memory to have another (new) record of StrGenericparameters structure
	TotParam= MyData.BaseParametersMine.TotParameters;
	TotParam++;
	SizeMem = TotParam * sizeof(StrGenericParameters);
	MyData.BaseParametersMine.pVetParameters = 
		(StrGenericParameters *) ResizeMemory(MyData.BaseParametersMine.pVetParameters, SizeMem);

	pMyParam = & MyData.BaseParametersMine.pVetParameters[TotParam-1];
	
	// now require memory for all arguments (NumberOfItems) store in pItemArray

	pMyParam->pVetArg = (short *) GetMemory(2 * NumberOfItems);
	// copy data
	pMyParam->NArguments = NumberOfItems;
	memcpy(pMyParam->pVetArg, pItemArray, 2*NumberOfItems);

	MyData.BaseParametersMine.TotParameters= TotParam;
	// ---- end of default managemnt for generic parameters -------------


}

// this procedure will be called every game cycle (at begin of cycle)
void cbCycleBegin(void)
{

}

// Not yet linked! To link it add to RequireMyCallBacks() function the row:
//  	GET_CALLBACK(CB_CYCLE_END, 0, 0, cbCycleEnd);
// this procedure will be called everygame cycle, at end.
// you have to return a RET_CYCLE_ value
int cbCycleEnd(void)
{


	return RET_CYCLE_CONTINUE;	
}

// this function will be called for each your (common) progressive action to be peformed
void PerformMyProgrAction(StrProgressiveAction *pAction)
{


	switch (pAction->ActionType) {
// replace the "case -1:" with your first "case AXN_...:" progressive action to manage)		
	case -1:
		break;

	}

}
// callback called from trng for each frame in game cycle to perform your (common) progressive action
void cbProgrActionMine(void)
{
	int i;
	StrProgressiveAction *pAction;

	pAction = &MyData.VetProgrActions[0];
	for (i=0;i<MyData.TotProgrActions;i++) {
		if (pAction->ActionType != AXN_FREE) {
			PerformMyProgrAction(pAction);
		}
		pAction++;
	}


}

bool TestSaveCrystalConditions(StrItemTr4 *Object, bool Restricted)
{
	StrItemTr4 *LaraObject;

	LaraObject = Trng.pGlobTomb4->pAdr->pLara;
	return *Trng.pGlobTomb4->pAdr->pInputGameCommands & CMD_ACTION
			&& (!Restricted || LaraObject->AnimationNow == ANIMATION_STAND_STILL_LINK || LaraObject->AnimationNow == ANIMATION_STAND_STILL_BREATH)
			&& *Trng.pGlobTomb4->pAdr->pFlagsLaraHands == FLH_ALMOST_FREE_HANDS
			&& MaximumDistance(LaraObject, Object) <= DISTANCE_CLOSE
			&& AbsoluteFacing(LaraObject, Object) <= FACING_RIGHT;
}

void InitialiseSaveCrystalBlue(short ItemIndex)
{
	StrItemTr4 *Object;

	Object = &Trng.pGlobTomb4->pAdr->pVetItems[ItemIndex];
	Object->StateIdCurrent = 0;
	Object->StateIdNext = 0;
	Object->AnimationNow = Trng.pGlobTomb4->pAdr->pVetSlot[Object->SlotID].IndexFirstAnim;
	Object->FrameNow = Trng.pGlobTomb4->pAdr->pVetAnimations[Object->AnimationNow].FrameStart;
	Object->FlagsMain = FITEM_CREATURE | FITEM_NOT_YET_ENABLED;
	Object->Objectbuttons |= MASK_ACTIVATION;
	AddActiveItem(ItemIndex);
}

void CollisionSaveCrystalBlue(short ItemIndex, StrItemTr4 *pLara, StrCollisionLara *pLaraCollision)
{
	StrItemTr4 *Object;

	Object = &Trng.pGlobTomb4->pAdr->pVetItems[ItemIndex];
	if (!TriggerActive(Object)
			|| MaximumDistance(pLara, Object) > DISTANCE_SECTOR
			|| !TestBoundCollide(Object, pLara, pLaraCollision->LaraSizeX)
			|| !TestCollision(Object, pLara))
		return;
	ItemPushLara(Object, pLara, pLaraCollision, false, 1);
}

void ControlSaveCrystalBlue(short ItemIndex)
{
	StrItemTr4 *Object;
	StrBoxCollisione *Collision;

	Object = &Trng.pGlobTomb4->pAdr->pVetItems[ItemIndex];
	AnimateItem(Object);
	Collision = GetBoundsAccurate(Object);
	TriggerDynamic(Object->CordX, Object->CordY + (Collision->MinY + Collision->MaxY) / 2, Object->CordZ, LIGHT_INTENSITY_SAVECRYSTAL, SaveCrystalBlueColour.Red, SaveCrystalBlueColour.Green, SaveCrystalBlueColour.Blue);
	if (TestSaveCrystalConditions(Object, SaveCrystalBluePreciseAnimation)) {
		MyData.Save.Global.SaveCrystalBlueIndex = ItemIndex;
		MyData.Save.Global.SendCommand = true;
	}
}

void InitialiseSaveCrystalGreen(short ItemIndex)
{
	StrItemTr4 *Object;

	Object = &Trng.pGlobTomb4->pAdr->pVetItems[ItemIndex];
	Object->StateIdCurrent = 0;
	Object->StateIdNext = 0;
	Object->AnimationNow = Trng.pGlobTomb4->pAdr->pVetSlot[Object->SlotID].IndexFirstAnim;
	Object->FrameNow = Trng.pGlobTomb4->pAdr->pVetAnimations[Object->AnimationNow].FrameStart;
	Object->FlagsMain = FITEM_CREATURE | FITEM_NOT_YET_ENABLED;
	Object->Objectbuttons |= MASK_ACTIVATION;
	AddActiveItem(ItemIndex);
}

void CollisionSaveCrystalGreen(short ItemIndex, StrItemTr4 *pLara, StrCollisionLara *pLaraCollision)
{
	StrItemTr4 *Object;

	Object = &Trng.pGlobTomb4->pAdr->pVetItems[ItemIndex];
	if (!TriggerActive(Object)
			|| MaximumDistance(pLara, Object) > DISTANCE_SECTOR
			|| !TestBoundCollide(Object, pLara, pLaraCollision->LaraSizeX)
			|| !TestCollision(Object, pLara))
		return;
	if (!SaveCrystalGreenPrecisePosition && MyData.Save.Global.SaveCrystalGreenQuantity < QUANTITY_SAVECRYSTAL_GREEN_MAXIMUM) {
		++MyData.Save.Global.SaveCrystalGreenQuantity;
		MyData.Save.Global.ClearCommand = !QuickSave;
		if (SaveCrystalGreenCollectSound != INDEX_NONE)
			SoundEffect(SaveCrystalGreenCollectSound, &Object->CordX, SFX_ALWAYS);
		KillItem(ItemIndex);
	}
	ItemPushLara(Object, pLara, pLaraCollision, false, 1);
}

void ControlSaveCrystalGreen(short ItemIndex)
{
	StrItemTr4 *Object;
	StrBoxCollisione *Collision;

	Object = &Trng.pGlobTomb4->pAdr->pVetItems[ItemIndex];
	AnimateItem(Object);
	Collision = GetBoundsAccurate(Object);
	TriggerDynamic(Object->CordX, Object->CordY + (Collision->MinY + Collision->MaxY) / 2, Object->CordZ, LIGHT_INTENSITY_SAVECRYSTAL, SaveCrystalGreenColour.Red, SaveCrystalGreenColour.Green, SaveCrystalGreenColour.Blue);
	if (SaveCrystalGreenPrecisePosition
			&& TestSaveCrystalConditions(Object, true)
			&& MyData.Save.Global.SaveCrystalGreenQuantity < QUANTITY_SAVECRYSTAL_GREEN_MAXIMUM) {
		++MyData.Save.Global.SaveCrystalGreenQuantity;
		MyData.Save.Global.ClearCommand = !QuickSave;
		if (SaveCrystalGreenCollectSound != INDEX_NONE)
			SoundEffect(SaveCrystalGreenCollectSound, &Object->CordX, SFX_ALWAYS);
		KillItem(ItemIndex);
	}
}

void InitialiseFallingFloor(short ItemIndex)
{
	StrItemTr4 *Object;

	Object = &Trng.pGlobTomb4->pAdr->pVetItems[ItemIndex];
	Object->StateIdCurrent = STATE_FALLING_FLOOR_WAIT;
	Object->StateIdNext = STATE_FALLING_FLOOR_WAIT;
	Object->AnimationNow = Trng.pGlobTomb4->pAdr->pVetSlot[Object->SlotID].IndexFirstAnim;
	Object->FrameNow = Trng.pGlobTomb4->pAdr->pVetAnimations[Object->AnimationNow].FrameStart;
	Object->FlagsMain = FITEM_NOT_YET_ENABLED;
}

void ControlFallingFloor(short ItemIndex)
{
	StrItemTr4 *Object, *LaraObject;
	short Room;
	StrBoxCollisione *Collision;

	Object = &Trng.pGlobTomb4->pAdr->pVetItems[ItemIndex];
	Collision = GetBoundsAccurate(Object);
	switch (Object->StateIdCurrent) {
	case STATE_FALLING_FLOOR_WAIT:
		LaraObject = Trng.pGlobTomb4->pAdr->pLara;
		if (LaraObject->CordY == Object->CordY + Collision->MinY
				&& SameSector(LaraObject->CordX, LaraObject->CordZ, Object->CordX, Object->CordZ)) {
			if (Object->Reserved_34 != TimeTolerance) {
				++Object->Reserved_34;
			} else {
				Room = Object->Room;
				Object->HeightFloor = GetHeight(GetFloor(Object->CordX, Object->CordY, Object->CordZ, &Room), Object->CordX, Object->CordY, Object->CordZ);
				Object->StateIdNext = STATE_FALLING_FLOOR_BREAK;
			}
		}
		break;
	case STATE_FALLING_FLOOR_FALL:
		if (Object->StateIdNext != STATE_FALLING_FLOOR_DIE) {
			Object->FlagsMain |= FITEM_GRAVITY_AFFECTED;
			Room = Object->Room;
			GetFloor(Object->CordX, Object->CordY, Object->CordZ, &Room);
			if (Object->Room != Room)
				ItemNewRoom(ItemIndex, Room);
			if (Object->CordY + Collision->MaxY > Object->HeightFloor) {
				Object->FlagsMain &= ~FITEM_GRAVITY_AFFECTED;
				Object->SpeedV = 0;
				Object->CordY = Object->HeightFloor - Collision->MaxY;
				Object->StateIdNext = STATE_FALLING_FLOOR_DIE;
			}
		} else {
			Object->CordY = Object->HeightFloor - Collision->MaxY;
		}
		break;
	case STATE_FALLING_FLOOR_DIE:
		Object->CordY = Object->HeightFloor - Collision->MaxY;
		break;
	}
	AnimateItem(Object);
}

void CeilingFallingFloor(StrItemTr4 *pItem, DWORD X, int Y, DWORD Z, int *pNewY)
{
	StrBoxCollisione *Collision;

	if ((pItem->StateIdCurrent == STATE_FALLING_FLOOR_WAIT || pItem->StateIdCurrent == STATE_FALLING_FLOOR_BREAK)
			&& SameSector(X, Z, pItem->CordX, pItem->CordZ)) {
		Collision = GetBoundsAccurate(pItem);
		if (Y > pItem->CordY + Collision->MinY)
			*pNewY = pItem->CordY + Collision->MinY + HEIGHT_MINIMUM;
	}
}

void FloorFallingFloor(StrItemTr4 *pItem, DWORD X, int Y, DWORD Z, int *pNewY)
{
	StrBoxCollisione *Collision;

	if ((pItem->StateIdCurrent == STATE_FALLING_FLOOR_WAIT || pItem->StateIdCurrent == STATE_FALLING_FLOOR_BREAK)
			&& SameSector(X, Z, pItem->CordX, pItem->CordZ)) {
		Collision = GetBoundsAccurate(pItem);
		if (Y < pItem->CordY + Collision->MinY) {
			*pNewY = pItem->CordY + Collision->MinY;
			*Trng.pGlobTomb4->pAdr->pTestOverPlatform = true;
			*Trng.pGlobTomb4->pAdr->pInclinationType = SLOPE_FLAT;
		}
	}
}

void InitialiseTR1Spikes(short ItemIndex)
{
	StrItemTr4 *Object;

	Object = &Trng.pGlobTomb4->pAdr->pVetItems[ItemIndex];
	Object->StateIdCurrent = 0;
	Object->StateIdNext = 0;
	Object->AnimationNow = Trng.pGlobTomb4->pAdr->pVetSlot[Object->SlotID].IndexFirstAnim;
	Object->FrameNow = Trng.pGlobTomb4->pAdr->pVetAnimations[Object->AnimationNow].FrameStart;
}

void CollisionTR1Spikes(short ItemIndex, StrItemTr4 *pLara, StrCollisionLara *pLaraCollision)
{
	StrItemTr4 *Object;
	StrBoxCollisione *Collision, *CollisionLara;
	int BloodCount, Top, TopLara, Bottom, BottomLara, Index;

	Object = &Trng.pGlobTomb4->pAdr->pVetItems[ItemIndex];
	if (MaximumDistance(pLara, Object) > DISTANCE_SECTOR
			|| !TestBoundCollide(Object, pLara, pLaraCollision->LaraSizeX)
			|| !TestCollision(Object, pLara))
		return;
	BloodCount = 0;
	if (*Trng.pGlobTomb4->pAdr->pLaraLocationFlags != LLF_DOZY) {
		if (pLara->FlagsMain & FITEM_GRAVITY_AFFECTED) {
			if (InstantKill && pLara->SpeedV > SPEED_VERTICAL_THRESHOLD) {
				pLara->Health = 0;
				BloodCount = BLOOD_COUNT_INSTANT_KILL;
			} else {
				if (HealthLossPerFrame) {
					pLara->Health -= HealthLossPerFrame;
					BloodCount = GetRandomControl() % (BLOOD_COUNT_MAXIMUM - BLOOD_COUNT_MINIMUM + 1) + BLOOD_COUNT_MINIMUM;
				}
			}
		} else {
			if (HealthLossPerFrame && pLara->SpeedH >= SPEED_HORIZONTAL_THRESHOLD) {
				pLara->Health -= HealthLossPerFrame;
				BloodCount = GetRandomControl() % (BLOOD_COUNT_MAXIMUM - BLOOD_COUNT_MINIMUM + 1) + BLOOD_COUNT_MINIMUM;
			}
		}
	}
	if (BloodCount > 0) {
		Collision = GetBoundsAccurate(Object);
		CollisionLara = GetBoundsAccurate(pLara);
		Bottom = Object->CordY + Collision->MaxY;
		BottomLara = pLara->CordY + CollisionLara->MaxY;
		Bottom = Bottom < BottomLara ? Bottom : BottomLara;
		Top = Object->CordY + Collision->MinY;
		TopLara = pLara->CordY + CollisionLara->MinY;
		Top = Top > TopLara ? Top : TopLara;
		for (Index = 0; Index < BloodCount; ++Index)
			TriggerBlood(pLara->CordX + GetRandomControl() % (2 * DISTANCE_TOUCH) - DISTANCE_TOUCH, Top + GetRandomControl() % (Bottom - Top), pLara->CordZ + GetRandomControl() % (2 * DISTANCE_TOUCH) - DISTANCE_TOUCH, GetRandomControl() + GetRandomControl(), 1);
		if (pLara->Health <= 0
				&& (ImpalementAfterHealthDepletion && BloodCount != BLOOD_COUNT_INSTANT_KILL || ImpalementAfterLanding && BloodCount == BLOOD_COUNT_INSTANT_KILL)) {
			pLara->SpeedH = 0;
			pLara->FlagsMain &= ~FITEM_GRAVITY_AFFECTED;
			pLara->AnimationNow = DeathAnimation;
			pLara->FrameNow = Trng.pGlobTomb4->pAdr->pVetAnimations[pLara->AnimationNow].FrameStart;
			pLara->StateIdCurrent = Trng.pGlobTomb4->pAdr->pVetAnimations[pLara->AnimationNow].StateId;
			pLara->StateIdNext = Trng.pGlobTomb4->pAdr->pVetAnimations[pLara->AnimationNow].StateId;
		}
	}
}

void InitSaveCrystalBlue(void)
{
	StrSlot *Slot;

	if (SaveCrystalBlueSlot) {
		if (PerformConditionTrigger(STRING_PLUGIN_CLASSIC_INVENTORY, INVENTORY_CONDITION_STATE, 0, 1)) {
			Slot = &Trng.pGlobTomb4->pAdr->pVetSlot[SaveCrystalBlueSlot];
			if (Slot->Flags & FSLOT_PRESENT) {
				Slot->pProcInitialise = InitialiseSaveCrystalBlue;
				Slot->pProcCollision = CollisionSaveCrystalBlue;
				Slot->pProcControl = ControlSaveCrystalBlue;
				Slot->Flags = FSLOT_SAVE_ALL_DATA | FSLOT_AFFECT_LARA_AT_CONTACT | FSLOT_SFX_LOCAL_SOUND | FSLOT_USE_COLLISION_BOX;
				return;
			}
		}
		SaveCrystalBlueSlot = 0;
	}
}

void InitSaveCrystalGreen(void)
{
	StrSlot *Slot;

	if (SaveCrystalGreenSlot) {
		if (PerformConditionTrigger(STRING_PLUGIN_CLASSIC_INVENTORY, INVENTORY_CONDITION_STATE, 0, 1)) {
			Slot = &Trng.pGlobTomb4->pAdr->pVetSlot[SaveCrystalGreenSlot];
			if (Slot->Flags & FSLOT_PRESENT) {
				Slot->pProcInitialise = InitialiseSaveCrystalGreen;
				Slot->pProcCollision = CollisionSaveCrystalGreen;
				Slot->pProcControl = ControlSaveCrystalGreen;
				Slot->Flags = FSLOT_SAVE_ALL_DATA | FSLOT_AFFECT_LARA_AT_CONTACT | FSLOT_SFX_LOCAL_SOUND | FSLOT_USE_COLLISION_BOX;
				return;
			}
		}
		SaveCrystalGreenSlot = 0;
	}
}

void InitFallingFloor(void)
{
	StrSlot *Slot;

	if (FallingFloorSlot) {
		Slot = &Trng.pGlobTomb4->pAdr->pVetSlot[FallingFloorSlot];
		if (Slot->Flags & FSLOT_PRESENT) {
			Slot->pProcInitialise = InitialiseFallingFloor;
			Slot->pProcCollision = NULL;
			Slot->pProcControl = ControlFallingFloor;
			Slot->pProcCeiling = CeilingFallingFloor;
			Slot->pProcFloor = FloorFallingFloor;
			Slot->Flags = FSLOT_CHANGE_POS_ITEM | FSLOT_SAVE_ALL_DATA | FSLOT_AFFECT_LARA_AT_CONTACT | FSLOT_SFX_LOCAL_SOUND | FSLOT_USE_COLLISION_BOX;
			return;
		}
		FallingFloorSlot = 0;
	}
}

void InitTR1Spikes(void)
{
	StrSlot *Slot;

	if (TR1SpikesSlot) {
		Slot = &Trng.pGlobTomb4->pAdr->pVetSlot[TR1SpikesSlot];
		if (Slot->Flags & FSLOT_PRESENT) {
			Slot->pProcInitialise = InitialiseTR1Spikes;
			Slot->pProcCollision = CollisionTR1Spikes;
			Slot->pProcControl = NULL;
			Slot->Flags = FSLOT_SAVE_ALL_DATA | FSLOT_AFFECT_LARA_AT_CONTACT | FSLOT_USE_COLLISION_BOX;
			return;
		}
		TR1SpikesSlot = 0;
	}
}

// inside this function you'll type call to functions to intialise your new objects or customize that olds.
// this callback will be called at start of loading new level and a bit after having started to load level data
void cbInitObjects(void) 
{
	InitSaveCrystalBlue();
	InitSaveCrystalGreen();
	InitFallingFloor();
	InitTR1Spikes();
}

bool cbInputManager(BYTE VetInputKeyboard[], WORD *pInputGameCommands, DWORD *pInputCommandFlags, DWORD *pInputExtGameCommands)
{
	if (!*Trng.pGlobTomb4->pAdr->pTestGrayScreen) {
		if (!MyData.Save.Global.Guard) {
			if (MyData.Save.Global.SendCommand) {
				VetInputKeyboard[KEY_F5] = 1;
				MyData.Save.Global.Guard = true;
			} else {
				if (MyData.Save.Global.ClearCommand)
					VetInputKeyboard[KEY_F5] = 0;
			}
		} else {
			MyData.Save.Global.Guard = false;
			MyData.Save.Global.SendCommand = false;
			VetInputKeyboard[KEY_F5] = 0;
		}
	}
	return SRET_OK;
}

void *cbNumericTrngPatch(WORD PatchIndex, WORD CBT_Flags, bool TestFromJump, StrRegisters *pRegisters)
{
	StrItemTr4 *Object;

	if ((SaveCrystalBlueSlot || SaveCrystalGreenSlot) && !strstr(*SavegameFilenameFormat, STRING_SAVEGAME_FILENAME_FORMAT)) {
		if (MyData.Save.Global.SendCommand) {
			Object = &Trng.pGlobTomb4->pAdr->pVetItems[MyData.Save.Global.SaveCrystalBlueIndex];
			if (SaveCrystalBlueUseSound != INDEX_NONE)
				SoundEffect(SaveCrystalBlueUseSound, &Object->CordX, SFX_ALWAYS);
			KillItem(MyData.Save.Global.SaveCrystalBlueIndex);
		} else {
			if (SaveCrystalGreenSlot && MyData.Save.Global.SaveCrystalGreenQuantity) {
				--MyData.Save.Global.SaveCrystalGreenQuantity;
				MyData.Save.Global.ClearCommand = !QuickSave || !MyData.Save.Global.SaveCrystalGreenQuantity;
			}
		}
	}
	return NULL;
}

// FOR_YOU:
// in this function RequireMyCallBacks() you'll type
// a list of:
//		GET_CALLBACK(CB_..., ,)
// one for each callback you need
bool RequireMyCallBacks(void)
{
// ************  RequireMyCallBacks() function  *****************
	// protype of GET_CALLBACK:
	// GET_CALLBACK(CallBackCB, CBT_Flags, Index, MyProcToCall)
	// default callbacks required always 
	GET_CALLBACK(CB_INIT_PROGRAM, 0, 0, cbInitProgram)
	GET_CALLBACK(CB_INIT_GAME, 0, 0, cbInitGame)
	GET_CALLBACK(CB_INIT_LEVEL, 0,0, cbInitLevel)
	GET_CALLBACK(CB_SAVING_GAME, 0, 0, cbSaveMyData)
	GET_CALLBACK(CB_LOADING_GAME, 0, 0, cbLoadMyData)
	GET_CALLBACK(CB_INIT_LOAD_NEW_LEVEL, 0,0, cbInitLoadNewLevel);
	GET_CALLBACK(CB_FLIPEFFECT_MINE, 0, 0, cbFlipEffectMine);
	GET_CALLBACK(CB_ACTION_MINE, 0,0, cbActionMine);
	GET_CALLBACK(CB_CONDITION_MINE,0,0,cbConditionMine);
	GET_CALLBACK(CB_CUSTOMIZE_MINE, 0,0, cbCustomizeMine);
	GET_CALLBACK(CB_PARAMETER_MINE, 0, 0, cbParametersMine);
	GET_CALLBACK(CB_ASSIGN_SLOT_MINE, 0,0, cbAssignSlotMine);
	GET_CALLBACK(CB_CYCLE_BEGIN, 0, 0, cbCycleBegin);
	GET_CALLBACK(CB_PROGR_ACTION_MINE, 0, 0, cbProgrActionMine);
	GET_CALLBACK(CB_INIT_OBJECTS, 0, 0, cbInitObjects);

	GET_CALLBACK(CB_INPUT_MANAGER, 0, 0, cbInputManager);
	GET_CALLBACK(CB_NUMERIC_TRNG_PATCH, CBT_FIRST, PATCH_GT_SAVED_SAVEGAME, cbNumericTrngPatch);

	return true;
}
// FOR_YOU:
// This function will be the first code to be executed of your plugin
// It happens when trng will load your plugin.
// In this moment no directX graphic is yet active, so you can show
// message boxes (TryMessageBox) to advise your players or level designer 
// about something
// Note: if you wish abort all (because there is an error or something
// is missing) you have to exit from this function returning: false
bool InitializeAll(void)
{
// ************  InitializeAll() function  ****************
	//  perform all your patches
	CALL_CHECK(CreateMyCodePatches)

	// call the function that requires all callback you need
	CALL_CHECK(RequireMyCallBacks)

	// TYPE_HERE: code to allocate global resource to use in the whole game

	return true;
}

// FOR_YOU: Tyis function will be called when tomb4 game is to be closed.
// you should type in this function further codes to free the global
// resource you had allocated in the InitializeAll() function 
void ReleaseAll(void)
{
// ************  ReleaseAll() function  ******************
	FreeLevelResources();
}


BOOL APIENTRY DllMain( HINSTANCE hInstanceDll, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved)
{

    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			MyDllInstance = hInstanceDll;
			GetTrngInfo();
			// control per check control value about size and alignment with globtomb4 structure
			if (CheckControlGlobTomb4() == false) return FALSE;

			if  (InitializeAll()==false) {
				return FALSE;
			}
			return TRUE;
			

		case DLL_PROCESS_DETACH:
			ReleaseAll();
			break;
    }
    return TRUE;
}


